
local VUHDO_SHIELD_LEFT = { };
local VUHDO_SHIELD_SIZE = { };
local VUHDO_SHIELD_EXPIRY = { };



--
local VUHDO_PLAYER_GUID = UnitGUID("player");
local pairs = pairs;
local ceil = ceil;
function VUHDO_shieldAbsorbInitBurst()
end



--
local function VUHDO_initShieldValue(aUnit, aShieldName, anAmount, aDuration)
	if ((anAmount or 0) == 0) then
		--VUHDO_xMsg("ERROR: Failed to init shield " .. aShieldName .. " on " .. aUnit, anAmount);
		return;
	end

	if (VUHDO_SHIELD_LEFT[aUnit] == nil) then
		VUHDO_SHIELD_LEFT[aUnit], VUHDO_SHIELD_SIZE[aUnit], VUHDO_SHIELD_EXPIRY[aUnit] = {}, {}, {};
	end

	VUHDO_SHIELD_LEFT[aUnit][aShieldName] = anAmount;
	VUHDO_SHIELD_SIZE[aUnit][aShieldName] = anAmount;
	VUHDO_SHIELD_EXPIRY[aUnit][aShieldName] = GetTime() + aDuration;
	--VUHDO_xMsg("Init shield " .. aShieldName .. " on " .. aUnit .. " for " .. anAmount);
end



--
local function VUHDO_updateShieldValue(aUnit, aShieldName, anAmount, aDuration)
	if (VUHDO_SHIELD_SIZE[aUnit] == nil or VUHDO_SHIELD_SIZE[aUnit][aShieldName] == nil) then
		return;
	end

	if (VUHDO_SHIELD_LEFT[aUnit][aShieldName] <= anAmount) then
		VUHDO_SHIELD_EXPIRY[aUnit][aShieldName] = GetTime() + aDuration;
		--VUHDO_Msg("Shield overwritten");
	end

	VUHDO_SHIELD_LEFT[aUnit][aShieldName] = anAmount;
	--VUHDO_Msg("Updated shield " .. aShieldName .. " on " .. aUnit .. " to " .. anAmount);
end



--
local function VUHDO_removeShield(aUnit, aShieldName)
	if (VUHDO_SHIELD_SIZE[aUnit] == nil or VUHDO_SHIELD_SIZE[aUnit][aShieldName] == nil) then
		return;
	end

	VUHDO_SHIELD_SIZE[aUnit][aShieldName] = nil;
	VUHDO_SHIELD_LEFT[aUnit][aShieldName] = nil;
	VUHDO_SHIELD_EXPIRY[aUnit][aShieldName] = nil;
	--VUHDO_Msg("Removed shield " .. aShieldName .. " from " .. aUnit);
end



--
local tNow;
local tUnit, tAllShields;
local tShieldName, tExpiry;
function VUHDO_removeObsoleteShields()
	tNow = GetTime();
	for tUnit, tAllShields in pairs(VUHDO_SHIELD_EXPIRY) do
		for tShieldName, tExpiry in pairs(tAllShields) do
			if (tExpiry < tNow) then
				VUHDO_removeShield(tUnit, tShieldName);
			end
		end
	end
end



--
local tInit;
local tEmptyShields = { };
function VUHDO_getShieldLeftCount(aUnit, aShield)
	tInit = (VUHDO_SHIELD_SIZE[aUnit] or tEmptyShields)[aShield] or 0;

	if (tInit > 0) then
		return ceil(4 * ((VUHDO_SHIELD_LEFT[aUnit] or tEmptyShields)[aShield] or 0) / tInit);
	else
		return 0;
	end
end





--
--[[local tAmountText, tAmount;
local function VUHDO_initPowerWordShieldAbsorb(aUnit, aBuffIndex)
	--VUHDO_xMsg("Init PW:S", aUnit, aBuffIndex);
  if (not VuhDoScanTooltip:IsOwned(VuhDo)) then
  	VuhDoScanTooltip:SetOwner(VuhDo, "ANCHOR_NONE");
  end
  VuhDoScanTooltip:ClearLines();
  VuhDoScanTooltip:SetUnitBuff(aUnit, aBuffIndex);
  tAmountText = VuhDoScanTooltipTextLeft2:GetText();
	tAmount = VUHDO_getNumbersFromString(tAmountText, 1)[1];
	VUHDO_initShieldValue(aUnit, VUHDO_SPELL_ID_POWERWORD_SHIELD, tAmount);
end



--
local tMultiplyer, tAmount;
local function VUHDO_initDivineAegisAbsorb(aUnit, _)
	--VUHDO_xMsg("Init Divine Aegis", aUnit);
	tMultiplyer = VUHDO_getDivineAegisSkill() * 0.1; -- 10/20/30% of last healing
	tAmount = VUHDO_getLastHealAmount() * tMultiplyer;
	VUHDO_initShieldValue(aUnit, VUHDO_SPELL_ID_DIVINE_AEGIS, tAmount);
end



--
VUHDO_SHIELDS = {
	[VUHDO_SPELL_ID_POWERWORD_SHIELD] = VUHDO_initPowerWordShieldAbsorb,
	[VUHDO_SPELL_ID_DIVINE_AEGIS] = VUHDO_initDivineAegisAbsorb,
};
]]



--
local VUHDO_SHIELDS = {
	[VUHDO_SPELL_ID_POWERWORD_SHIELD] = 30,
	[VUHDO_SPELL_ID_DIVINE_AEGIS] = 12,
	--[VUHDO_SPELL_ID_POWER_WORD_BARRIER] = 25,
};



--
local tUnit;
function VUHDO_parseCombatLogShieldAbsorb(aMessage, aSrcGuid, aDstGuid, aShieldName, anAmount)
	if (VUHDO_PLAYER_GUID ~= aSrcGuid or VUHDO_SHIELDS[aShieldName] == nil) then
		return;
	end

	tUnit = VUHDO_RAID_GUIDS[aDstGuid];
	if (tUnit == nil) then
		return;
	end

	if ("SPELL_AURA_REFRESH" == aMessage) then
		VUHDO_updateShieldValue(tUnit, aShieldName, anAmount, VUHDO_SHIELDS[aShieldName]);
	elseif ("SPELL_AURA_APPLIED" == aMessage) then
		VUHDO_initShieldValue(tUnit, aShieldName, anAmount, VUHDO_SHIELDS[aShieldName]);
	elseif ("SPELL_AURA_REMOVED" == aMessage) then
		VUHDO_removeShield(tUnit, aShieldName);
	end
end
